@@ -10,9 +10,9 @@
 #define WORDSCHUNK 8
 #define COLSCHUNK 16
 
-#define TPARM1(p, ...) tparm(p, 0, 0, 0, 0, 0, 0, 0, 0, 0)
-#define TPARM2(p, q, ...) tparm(p, q, 0, 0, 0, 0, 0, 0, 0, 0)
-#define TPARM3(p, q, r, ...) tparm(p, q, r, 0, 0, 0, 0, 0, 0, 0)
+#define TPARM1(p) tparm(p, 0, 0, 0, 0, 0, 0, 0, 0, 0)
+#define TPARM2(p, q) tparm(p, q, 0, 0, 0, 0, 0, 0, 0, 0)
+#define TPARM3(p, q, r) tparm(p, q, r, 0, 0, 0, 0, 0, 0, 0)
 
 #define _XOPEN_SOURCE 700
 
@@ -20,7 +20,7 @@
 /* """""""""""""""""""""""""" */
 #define SECOND 1000000
 #define FREQ 10
-#define TICK (SECOND / FREQ)
+#define TCK (SECOND / FREQ)
 
 /* Bit array management */
 /* """""""""""""""""""" */
@@ -51,229 +51,68 @@ typedef struct output_s      output_t;
 typedef struct daccess_s     daccess_t;
 typedef struct search_data_s search_data_t;
 
-typedef enum filter_types       filters_t;
-typedef enum daccess_modes      da_mode_t;
-typedef enum timeout_modes      to_mode_t;
-typedef enum attribute_settings attr_set_t;
-typedef enum search_modes       search_mode_t;
-typedef enum bitmap_affinities  bitmap_affinity_t;
-
-/* ********** */
-/* Prototypes */
-/* ********** */
-
-void
-help(win_t * win, term_t * term, long last_line, toggle_t * toggle);
-
-int
-tag_comp(void * a, void * b);
-
-void
-tag_swap(void * a, void * b);
-
-int
-isempty(const char * s);
-
-void
-beep(toggle_t * toggle);
-
-int
-get_cursor_position(int * const r, int * const c);
-
-void
-get_terminal_size(int * const r, int * const c);
-
-int
-#ifdef __sun
-outch(char c);
-#else
-outch(int c);
-#endif
-
-void
-restore_term(int const fd);
-
-void
-setup_term(int const fd);
-
-void
-strip_ansi_color(char * s, toggle_t * toggle);
-
-int
-tst_cb(void * elem);
-
-int
-tst_cb_cli(void * elem);
-
-int
-ini_load(const char * filename, win_t * win, term_t * term, limits_t * limits,
-         timers_t * timers, misc_t * misc,
-         int (*report)(win_t * win, term_t * term, limits_t * limits,
-                       timers_t * timers, misc_t * misc, const char * section,
-                       const char * name, char * value));
-
-int
-ini_cb(win_t * win, term_t * term, limits_t * limits, timers_t * timers,
-       misc_t * misc, const char * section, const char * name, char * value);
-
-char *
-make_ini_path(char * name, char * base);
-
-void
-set_foreground_color(term_t * term, short color);
-
-void
-set_background_color(term_t * term, short color);
-
-void
-set_win_start_end(win_t * win, long current, long last);
-
-long
-build_metadata(term_t * term, long count, win_t * win);
-
-long
-disp_lines(win_t * win, toggle_t * toggle, long current, long count,
-           search_mode_t search_mode, search_data_t * search_data,
-           term_t * term, long last_line, char * tmp_word,
-           langinfo_t * langinfo);
-
-void
-get_message_lines(char * message, ll_t * message_lines_list,
-                  long * message_max_width, long * message_max_len);
-
-void
-disp_message(ll_t * message_lines_list, long width, long max_len, term_t * term,
-             win_t * win);
-
-void
-update_bitmaps(search_mode_t search_mode, search_data_t * search_data,
-               bitmap_affinity_t affinity);
-
-long
-find_next_matching_word(long * array, long nb, long value, long * index);
-
-long
-find_prev_matching_word(long * array, long nb, long value, long * index);
-
-void
-clean_matches(search_data_t * search_data, long size);
-
-void
-disp_cursor_word(long pos, win_t * win, term_t * term, int err);
-
-void
-disp_matching_word(long pos, win_t * win, term_t * term, int is_cursor,
-                   int err);
-
-void
-disp_word(long pos, search_mode_t search_mode, search_data_t * search_data,
-          term_t * term, win_t * win, char * tmp_word);
-
-size_t
-expand(char * src, char * dest, langinfo_t * langinfo, toggle_t * toggle);
-
-int
-get_bytes(FILE * input, char * utf8_buffer, langinfo_t * langinfo);
-
-int
-get_scancode(unsigned char * s, size_t max);
-
-char *
-get_word(FILE * input, ll_t * word_delims_list, ll_t * record_delims_list,
-         char * utf8_buffer, unsigned char * is_last, toggle_t * toggle,
-         langinfo_t * langinfo, win_t * win, limits_t * limits);
-
-void
-left_margin_putp(char * s, term_t * term, win_t * win);
-
-void
-right_margin_putp(char * s1, char * s2, langinfo_t * langinfo, term_t * term,
-                  win_t * win, long line, long offset);
-
-void
-sig_handler(int s);
-
-void
-set_new_first_column(win_t * win, term_t * term);
-
-int
-parse_sed_like_string(sed_t * sed);
-
-void
-parse_selectors(char * str, filters_t * filter, char * unparsed,
-                ll_t ** inc_interval_list, ll_t ** inc_regex_list,
-                ll_t ** exc_interval_list, ll_t ** exc_regex_list,
-                langinfo_t * langinfo);
-
-int
-replace(char * orig, sed_t * sed);
-
-int
-decode_attr_toggles(char * s, attr_t * attr);
-
-int
-parse_attr(char * str, attr_t * attr, short max_color);
-
-void
-apply_attr(term_t * term, attr_t attr);
-
-int
-delims_cmp(const void * a, const void * b);
-
-long
-get_line_last_word(long line, long last_line);
-
-void
-move_left(win_t * win, term_t * term, toggle_t * toggle,
-          search_data_t * search_data, langinfo_t * langinfo, long * nl,
-          long last_line, char * tmp_word);
-
-void
-move_right(win_t * win, term_t * term, toggle_t * toggle,
-           search_data_t * search_data, langinfo_t * langinfo, long * nl,
-           long last_line, char * tmp_word);
-
-int
-find_best_word_upward(long last_word, long s, long e);
-
-int
-find_best_word_downward(long last_word, long s, long e);
-
-void
-move_up(win_t * win, term_t * term, toggle_t * toggle,
-        search_data_t * search_data, langinfo_t * langinfo, long * nl,
-        long page, long first_selectable, long last_line, char * tmp_word);
-
-void
-move_down(win_t * win, term_t * term, toggle_t * toggle,
-          search_data_t * search_data, langinfo_t * langinfo, long * nl,
-          long page, long last_selectable, long last_line, char * tmp_word);
-
-/* ***************** */
-/* Emums and structs */
-/* ***************** */
+/* ***** */
+/* Enums */
+/* ***** */
 
 /* Various filter pseudo constants */
 /* """"""""""""""""""""""""""""""" */
-enum filter_types
+typedef enum filter_types
 {
   UNKNOWN_FILTER,
   INCLUDE_FILTER,
   EXCLUDE_FILTER
-};
+} filters_t;
 
 /* Used by the -N -F and -D options */
 /* """""""""""""""""""""""""""""""" */
-enum daccess_modes
+typedef enum daccess_modes
 {
   DA_TYPE_NONE = 0, /* must be 0 (boolean value) */
   DA_TYPE_AUTO = 1,
   DA_TYPE_POS  = 2
-};
+} da_mode_t;
+
+/* Various timeout mode used by the -x/-X option */
+/* """"""""""""""""""""""""""""""""""""""""""""" */
+typedef enum timeout_modes
+{
+  CURRENT, /* on timeout, outputs the selected word       */
+  QUIT,    /* on timeout, quit without selecting anything */
+  WORD     /* on timeout , outputs the specified word     */
+} to_mode_t;
+
+/* Constants used to set the color attributes */
+/* """""""""""""""""""""""""""""""""""""""""" */
+typedef enum attribute_settings
+{
+  UNSET = 0, /* must be 0 for future testings */
+  SET,
+  FORCED /* an attribute setting has been given in the command line */
+} attr_set_t;
+
+/* Constant to distinguish between the various search modes */
+/* """""""""""""""""""""""""""""""""""""""""""""""""""""""" */
+typedef enum search_modes
+{
+  NONE,
+  PREFIX,
+  FUZZY,
+  SUBSTRING
+} search_mode_t;
+
+/* Constants used in search mode to orient the bit-mask building */
+/* """"""""""""""""""""""""""""""""""""""""""""""""""""""""""""" */
+typedef enum bitmap_affinities
+{
+  NO_AFFINITY,
+  END_AFFINITY,
+  START_AFFINITY
+} bitmap_affinity_t;
 
 /* Used when managing the -R option */
 /* """""""""""""""""""""""""""""""" */
-enum row_regex_types
+enum
 {
   ROW_REGEX_EXCLUDE = 0, /* must be 0 (boolean value) */
   ROW_REGEX_INCLUDE = 1
@@ -281,11 +120,11 @@ enum row_regex_types
 
 /* Used when managing the -C option */
 /* """""""""""""""""""""""""""""""" */
-enum filter_infos
+enum
 {
-  EXCLUDE_MARK = 0,      /* must be 0 because used in various tests     *
+  EXCLUDE_MARK      = 0, /* must be 0 because used in various tests     *
                           * these words cannot be re-included           */
-  INCLUDE_MARK = 1,      /* to forcibly include a word, these words can *
+  INCLUDE_MARK      = 1, /* to forcibly include a word, these words can *
                           * be excluded later                           */
   SOFT_EXCLUDE_MARK = 2, /* word with this mark are excluded by default *
                           * but can be included later                   */
@@ -293,42 +132,9 @@ enum filter_infos
                           * but can be excluded later                   */
 };
 
-/* Various timeout mode used by the -x/-X option */
-/* """"""""""""""""""""""""""""""""""""""""""""" */
-enum timeout_modes
-{
-  CURRENT, /* on timeout, outputs the selected word       */
-  QUIT,    /* on timeout, quit without selecting anything */
-  WORD     /* on timeout , outputs the specified word     */
-};
-
-/* Constants used to set the color attributes */
-/* """""""""""""""""""""""""""""""""""""""""" */
-enum attribute_settings
-{
-  UNSET = 0, /* must be 0 for future testings */
-  SET,
-  FORCED /* an attribute setting has been given in the command line */
-};
-
-/* Constant to distinguish between the various search modes */
-/* """""""""""""""""""""""""""""""""""""""""""""""""""""""" */
-enum search_modes
-{
-  NONE,
-  PREFIX,
-  FUZZY,
-  SUBSTRING
-};
-
-/* Constants used in search mode to orient the bit-mask building */
-/* """"""""""""""""""""""""""""""""""""""""""""""""""""""""""""" */
-enum bitmap_affinities
-{
-  NO_AFFINITY,
-  END_AFFINITY,
-  START_AFFINITY
-};
+/* ******* */
+/* Structs */
+/* ******* */
 
 /* Used to store the different allowed charsets data */
 /* """"""""""""""""""""""""""""""""""""""""""""""""" */
@@ -381,6 +187,7 @@ struct timers_s
 struct misc_s
 {
   search_mode_t default_search_method;
+  char          invalid_char_substitute;
 };
 
 /* Terminal setting variables */
@@ -390,11 +197,12 @@ struct termios old_in_attrs;
 
 /* Interval timers used */
 /* """""""""""""""""""" */
-struct itimerval periodic_itv; /* refresh rate for the timeout counter */
+struct itimerval periodic_itv; /* refresh rate for the timeout counter. */
 
 int help_timer    = -1;
 int winch_timer   = -1;
 int daccess_timer = -1;
+int search_timer  = -1;
 
 /* Structure containing the attributes components */
 /* """""""""""""""""""""""""""""""""""""""""""""" */
@@ -447,25 +255,25 @@ struct term_s
 /* """"""""""""""""""""""""""" */
 struct word_s
 {
-  long start, end;      /* start/end absolute horiz. word positions *
-                         * on the screen                            */
-  size_t mb;            /* number of UTF-8 glyphs to display        */
-  long   tag_order;     /* each time a word is tagged, this value   *
-                         * is increased                             */
-  size_t special_level; /* can vary from 0 to 5; 0 meaning normal   */
-  char * str;           /* display string associated with this word */
-  size_t len;           /* number of bytes of str (for trimming)    */
-  char * orig;          /* NULL or original string if is had been   *
-                         * shortened for being displayed or altered *
-                         * by is expansion.                         */
-  char * bitmap;        /* used to store the the position of the    *
-                         * currently searched chars in a word. The  *
-                         * objective is to speed their display      */
+  long          start, end;    /* start/end absolute horiz. word positions *
+                                * on the screen                            */
+  size_t        mb;            /* number of UTF-8 glyphs to display        */
+  long          tag_order;     /* each time a word is tagged, this value   *
+                                * is increased                             */
+  size_t        special_level; /* can vary from 0 to 5; 0 meaning normal   */
+  char *        str;           /* display string associated with this word */
+  size_t        len;           /* number of bytes of str (for trimming)    */
+  char *        orig;          /* NULL or original string if is had been   *
+                                * shortened for being displayed or altered *
+                                * by is expansion.                         */
+  char *        bitmap;        /* used to store the the position of the    *
+                                * currently searched chars in a word. The  *
+                                * objective is to speed their display      */
   unsigned char is_matching;
-  unsigned char is_tagged;     /* 1 if the word is tagged, 0 if not   */
-  unsigned char is_last;       /* 1 if the word is the last of a line */
-  unsigned char is_selectable; /* word is selectable                  */
-  unsigned char is_numbered;   /* word has a direct access index      */
+  unsigned char is_tagged;     /* 1 if the word is tagged, 0 if not        */
+  unsigned char is_last;       /* 1 if the word is the last of a line      */
+  unsigned char is_selectable; /* word is selectable                       */
+  unsigned char is_numbered;   /* word has a direct access index           */
 };
 
 /* Structure describing the window in which the user */
@@ -473,22 +281,22 @@ struct word_s
 /* """"""""""""""""""""""""""""""""""""""""""""""""" */
 struct win_s
 {
-  long start, end;      /* index of the first and last word        */
-  long first_column;    /* number of the first character displayed */
-  long cur_line;        /* relative number of the cursor line (1+) */
-  long asked_max_lines; /* requested number of lines in the window */
-  long max_lines;       /* effective number of lines in the window */
-  long max_cols;        /* max number of words in a single line    */
-  long real_max_width;  /* max line length. In column, tab or line *
-                         * mode it can be greater than the         *
-                         * terminal width                          */
-  long message_lines;   /* number of lines taken by the messages   *
-                         * (updated by disp_message                */
-  long    max_width;    /* max usable line width or the terminal   */
-  long    offset;       /* window offset user when centered        */
-  char *  sel_sep;      /* output separator when tags are enabled  */
-  char ** gutter_a;     /* array of UTF-8 gutter glyphs            */
-  long    gutter_nb;    /* number of UTF-8 gutter glyphs           */
+  long    start, end;      /* index of the first and last word        */
+  long    first_column;    /* number of the first character displayed */
+  long    cur_line;        /* relative number of the cursor line (1+) */
+  long    asked_max_lines; /* requested number of lines in the window */
+  long    max_lines;       /* effective number of lines in the window */
+  long    max_cols;        /* max number of words in a single line    */
+  long    real_max_width;  /* max line length. In column, tab or line *
+                            * mode it can be greater than the         *
+                            * terminal width                          */
+  long    message_lines;   /* number of lines taken by the messages   *
+                            * (updated by disp_message                */
+  long    max_width;       /* max usable line width or the terminal   */
+  long    offset;          /* window offset user when centered        */
+  char *  sel_sep;         /* output separator when tags are enabled  */
+  char ** gutter_a;        /* array of UTF-8 gutter glyphs            */
+  long    gutter_nb;       /* number of UTF-8 gutter glyphs           */
 
   unsigned char tab_mode;  /* -t */
   unsigned char col_mode;  /* -c */
@@ -529,7 +337,7 @@ struct sed_s
                                *              occur more than once */
   unsigned char stop;         /* stop flag:   only one alteration  *
                                *              per word is allowed  */
-  regex_t re;                 /* compiled regular expression       */
+  regex_t       re;           /* compiled regular expression       */
 };
 
 /* Structure used to keep track of the different timeout values */
@@ -565,12 +373,13 @@ struct daccess_s
   int    length;     /* selector size (5 max)                             */
   int    flength;    /* 0 or length + 3 (full prefix lengh                */
   size_t offset;     /* offset to the start of the selector               */
+  char   missing;    /* y: number missing embedded numbers                */
   int    plus;       /* 1 if we can look for the number to extract after  *
                       * the offset, else 0. (a '+' follows the offset)    */
   int    size;       /* size in bytes of the selector to extract          */
   size_t ignore;     /* number of UTF-8 glyphs to ignore after the number */
-  char   follow;     /* y: the numbering follows the last nuber set       */
-  char * num_sep;    /* character to separate de number and the selection */
+  char   follow;     /* y: the numbering follows the last number set      */
+  char * num_sep;    /* character to separate number and selection        */
   int    def_number; /* 1: the numbering is on by default 0: it is not    */
 };
 
@@ -595,4 +404,202 @@ struct search_data_s
   int only_starting; /* same with the pattern at the beginning   */
 };
 
+/* ********** */
+/* Prototypes */
+/* ********** */
+
+void
+help(win_t * win, term_t * term, long last_line, toggle_t * toggle);
+
+int
+tag_comp(void * a, void * b);
+
+void
+tag_swap(void * a, void * b);
+
+int
+isempty(const char * s);
+
+void
+my_beep(toggle_t * toggle);
+
+int
+get_cursor_position(int * const r, int * const c);
+
+void
+get_terminal_size(int * const r, int * const c);
+
+int
+#ifdef __sun
+outch(char c);
+#else
+outch(int c);
+#endif
+
+void
+restore_term(int const fd);
+
+void
+setup_term(int const fd);
+
+void
+strip_ansi_color(char * s, toggle_t * toggle, misc_t * misc);
+
+int
+tst_cb(void * elem);
+
+int
+tst_cb_cli(void * elem);
+
+int
+ini_load(const char * filename, win_t * win, term_t * term, limits_t * limits,
+         timers_t * timers, misc_t * misc, langinfo_t * langinfo,
+         int (*report)(win_t * win, term_t * term, limits_t * limits,
+                       timers_t * timers, misc_t * misc, langinfo_t * langinfo,
+                       const char * section, const char * name, char * value));
+
+int
+ini_cb(win_t * win, term_t * term, limits_t * limits, timers_t * timers,
+       misc_t * misc, langinfo_t * langinfo, const char * section,
+       const char * name, char * value);
+
+char *
+make_ini_path(char * name, char * base);
+
+void
+set_foreground_color(term_t * term, short color);
+
+void
+set_background_color(term_t * term, short color);
+
+void
+set_win_start_end(win_t * win, long current, long last);
+
+long
+build_metadata(term_t * term, long count, win_t * win);
+
+long
+disp_lines(win_t * win, toggle_t * toggle, long current, long count,
+           search_mode_t search_mode, search_data_t * search_data,
+           term_t * term, long last_line, char * tmp_word,
+           langinfo_t * langinfo);
+
+void
+get_message_lines(char * message, ll_t * message_lines_list,
+                  long * message_max_width, long * message_max_len);
+
+void
+disp_message(ll_t * message_lines_list, long width, long max_len, term_t * term,
+             win_t * win);
+
+int
+check_integer_constraint(int nb_args, char ** args, char * value, char * par);
+
+void
+update_bitmaps(search_mode_t search_mode, search_data_t * search_data,
+               bitmap_affinity_t affinity);
+
+long
+find_next_matching_word(long * array, long nb, long value, long * index);
+
+long
+find_prev_matching_word(long * array, long nb, long value, long * index);
+
+void
+clean_matches(search_data_t * search_data, long size);
+
+void
+disp_cursor_word(long pos, win_t * win, term_t * term, int err);
+
+void
+disp_matching_word(long pos, win_t * win, term_t * term, int is_cursor,
+                   int err);
+
+void
+disp_word(long pos, search_mode_t search_mode, search_data_t * search_data,
+          term_t * term, win_t * win, char * tmp_word);
+
+size_t
+expand(char * src, char * dest, langinfo_t * langinfo, toggle_t * toggle,
+       misc_t * misc);
+
+int
+get_bytes(FILE * input, char * utf8_buffer, ll_t * ignored_glyphs_list,
+          langinfo_t * langinfo, misc_t * misc);
+
+int
+get_scancode(unsigned char * s, size_t max);
+
+char *
+get_word(FILE * input, ll_t * word_delims_list, ll_t * record_delims_list,
+         ll_t * ignored_glyphs_list, char * utf8_buffer,
+         unsigned char * is_last, toggle_t * toggle, langinfo_t * langinfo,
+         win_t * win, limits_t * limits, misc_t * misc);
+
+void
+left_margin_putp(char * s, term_t * term, win_t * win);
+
+void
+right_margin_putp(char * s1, char * s2, langinfo_t * langinfo, term_t * term,
+                  win_t * win, long line, long offset);
+
+void
+sig_handler(int s);
+
+void
+set_new_first_column(win_t * win, term_t * term);
+
+int
+parse_sed_like_string(sed_t * sed);
+
+void
+parse_selectors(char * str, filters_t * filter, char * unparsed,
+                ll_t ** inc_interval_list, ll_t ** inc_regex_list,
+                ll_t ** exc_interval_list, ll_t ** exc_regex_list,
+                langinfo_t * langinfo, misc_t * misc);
+
+int
+replace(char * orig, sed_t * sed);
+
+int
+decode_attr_toggles(char * s, attr_t * attr);
+
+int
+parse_attr(char * str, attr_t * attr, short max_color);
+
+void
+apply_attr(term_t * term, attr_t attr);
+
+int
+delims_cmp(const void * a, const void * b);
+
+long
+get_line_last_word(long line, long last_line);
+
+void
+move_left(win_t * win, term_t * term, toggle_t * toggle,
+          search_data_t * search_data, langinfo_t * langinfo, long * nl,
+          long last_line, char * tmp_word);
+
+void
+move_right(win_t * win, term_t * term, toggle_t * toggle,
+           search_data_t * search_data, langinfo_t * langinfo, long * nl,
+           long last_line, char * tmp_word);
+
+int
+find_best_word_upward(long last_word, long s, long e);
+
+int
+find_best_word_downward(long last_word, long s, long e);
+
+void
+move_up(win_t * win, term_t * term, toggle_t * toggle,
+        search_data_t * search_data, langinfo_t * langinfo, long * nl,
+        long page, long first_selectable, long last_line, char * tmp_word);
+
+void
+move_down(win_t * win, term_t * term, toggle_t * toggle,
+          search_data_t * search_data, langinfo_t * langinfo, long * nl,
+          long page, long last_selectable, long last_line, char * tmp_word);
+
 #endif
