@@ -59,4 +59,10 @@ isprint7(int i);
 int
 isprint8(int i);
 
+int
+xwcscasecmp(const wchar_t * w1s, const wchar_t * w2s);
+
+int
+is_integer(const char * const s);
+
 #endif
